package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.AlivenessDao;
import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;

import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.util.AuditLogger;

import dk.nsp.idsas.util.health.DBHealthIndicator;
import dk.nsp.idsas.util.health.HealthIndicator;
import dk.nsp.idsas.util.health.VersionIndicator;

import dk.sds.nsp.security.Security;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;
import org.springframework.jdbc.datasource.lookup.JndiDataSourceLookup;

import javax.sql.DataSource;

@Configuration
public class ServiceSetup {
    public static final String ANNOTATION_CONFIG_CONTEXT = "ANNOTATION_CONFIG_APPLICATION_CONTEXT";

    @Value("${version}") // From pom.properties
    private String version;

    @Value("${idsas.datasource.jndi}")
    private String dataSourceName;
    
    @Value("${personinformation.url}")
    private String personInformationUrl;

    @Value("${personinformation.errorcount.duration:PT10M}")
    private String personInformationErrorCountDuration;

    @Value("${sores.url}")
    private String soresUrl;

    @Value("${sores.errorcount.duration:PT10M}")
    private String soresErrorCountDuration;

    @Bean
    public DataSource dataSource() {
        return new JndiDataSourceLookup().getDataSource(dataSourceName);
    }

    @Bean
    public AlivenessDao alivenessDao(DataSource dataSource) {
        return new AlivenessDao(dataSource);
    }

    @Bean
    WhitelistDao whitelistDao(DataSource dataSource) {
        return new WhitelistDao(dataSource);
    }

    @Bean
    public BlurringDao blurringDao(DataSource dataSource) {
        return new BlurringDao(dataSource);
    }

    @Bean
    public SaltDao saltDao(DataSource dataSource) {
        return new SaltDao(dataSource);
    }

    @Bean
    @Lazy // Not used in idsas-operations-web
    public IdsasActorProvider idsasActorProvider() {
        return new IdsasActorProviderImpl(Security::getSecurityContext);
    }

    @Bean
    @Lazy // Not used in idsas-operations-web
    public AuditLogger auditLogger() {
        return new AuditLogger();
    }

    @Bean
    public HealthIndicator versionIndicator() {
        return new VersionIndicator(version);
    }

    @Bean
    public DBHealthIndicator dbHealthIndicator(AlivenessDao alivenessDao) {
        return new DBHealthIndicator(alivenessDao);
    }
}
