package dk.nsp.idsas.ws;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * Servlet for loading service contract.
 * Example path: secure-wsdl/20230601/idsas-lookup.wsdl
 */
public abstract class AbstractWsdlServlet extends AbstractServlet {
    private final Map<String, byte[]> resources = new HashMap<>();

    abstract List<String> validSuffixes();

    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String path = Optional.ofNullable(request.getPathInfo()).orElse("");

        Optional<byte[]> content = getResource(path);
        if (!content.isPresent()) {
            response.setStatus(404);
            return;
        }

        prepareResponse(response, content.get());
    }

    private Optional<byte[]> getResource(String path) throws IOException {
        if (validSuffixes().stream().noneMatch(path::endsWith)) {
            return Optional.empty();
        }
        if (!resources.containsKey(path)) {
            try (InputStream is = getClass().getResourceAsStream("/" + path)) {
                if (is != null) {
                    byte[] content = new byte[is.available()];
                    is.read(content);
                    resources.put(path, content);
                }
            }
        }
        return Optional.ofNullable(resources.get(path));
    }

    private void prepareResponse(HttpServletResponse response, byte[] content) throws IOException {
        response.setContentType("text/xml; charset=utf-8");
        response.setContentLength(content.length);
        response.getOutputStream().write(content);
        response.setStatus(200);
    }
}
