package dk.nsp.idsas.util;

import dk.sds.nsp.audit.AuditBuilder;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.time.Instant;
import java.util.Date;
import java.util.GregorianCalendar;
public class AuditLoggerTest {

    private AuditBuilder auditBuilder;
    private AuditLogger auditLogger;

    @Before
    public void setUp() {
        auditBuilder = Mockito.mock(AuditBuilder.class);
        auditLogger = new AuditLogger(() -> auditBuilder);
    }

    @Test
    public void logCreateBlurring_logsCprAndCvr() {
        String cpr = "1111111111";
        String cvr = "1111";
        auditLogger.logCreateBlurring(cpr, cvr);

        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.RegularPersonalInformation, "cpr", cpr);
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "createBlurring", AuditBuilder.TypeOfInformation.NonPersonalInformation, "cvr", cvr);
        Mockito.verifyNoMoreInteractions(auditBuilder);
    }

    @Test
    public void logGetSalt_logsCvrAndDateTime() throws DatatypeConfigurationException {
        String cvr = "2222";
        XMLGregorianCalendar dateTime = toXmlGregorian(Date.from(Instant.now()));
        auditLogger.logGetSalt(cvr, dateTime);

        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getSalt", AuditBuilder.TypeOfInformation.NonPersonalInformation, "cvr", cvr);
        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getSalt", AuditBuilder.TypeOfInformation.NonPersonalInformation, "dateTime", dateTime);
        Mockito.verifyNoMoreInteractions(auditBuilder);
    }

    @Test
    public void logGetCurrentSalt_logsCvr() {
        String cvr = "3333";
        auditLogger.logGetCurrentSalt(cvr);

        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getCurrentSalt", AuditBuilder.TypeOfInformation.NonPersonalInformation, "cvr", cvr);
        Mockito.verifyNoMoreInteractions(auditBuilder);
    }

    @Test
    public void logGetBlurredOrganisations_logsCpr() {
        String cpr = "2222222222";
        auditLogger.logGetBlurredOrganisations(cpr);

        Mockito.verify(auditBuilder).addAuditInformation("IDSAS", "getBlurredOrganisations", AuditBuilder.TypeOfInformation.RegularPersonalInformation, "cpr", cpr);
        Mockito.verifyNoMoreInteractions(auditBuilder);
    }

    private XMLGregorianCalendar toXmlGregorian(Date date) throws DatatypeConfigurationException {
        GregorianCalendar gregorianCalendar = (GregorianCalendar) GregorianCalendar.getInstance();
        gregorianCalendar.setTime(date);
        return DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
    }
}