package dk.nsp.idsas.test.stepdefinitions;

import dk.nsp.idsas.test.idptest.SOAPOperation;
import dk.nsp.idsas.test.idptest.ServiceCaller;
import dk.nsp.idsas.test.util.TestIdentities;
import dk.nsp.test.idp.OrganizationIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.GetCurrentSaltRequest;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.apache.commons.codec.binary.Hex;
import org.springframework.beans.factory.annotation.Value;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;


public class SaltStepDefinitions {

    @Value("${idsas.service.endpoint}" + "/20230601/service")
    private String serviceEndpoint;

    private final ServiceCaller serviceCaller = new ServiceCaller();

    private final List<String> responses = new ArrayList<>();
    private final List<Boolean> requestFailedList = new ArrayList<>();

    @When("en {string} forespørger salt")
    public void enXForespørgerSalt(String userType) {
        Identity identity = TestIdentities.identityFor(userType);
        sendGetCurrentSaltRequest(identity);
    }

    @When("en ikke whitelisted systembruger forespørger salt")
    public void enIkkeWhitelistedSystembrugerForespørgerSalt() {
        Identity identity = OrganizationIdentities.testOrganisation98021838();
        sendGetCurrentSaltRequest(identity);
    }

    @Then("returneres et salt")
    public void returneresEtSalt() {
        validateSaltReturnedForRequestX(0);
    }

    @Then("fejler forespørgslen på salt")
    public void fejlerForespørgslen() {
        validateRequestXFailed(0);
    }

    @Then("der returneres nyt salt på anden forespørgsel")
    public void derReturneresNytSaltPåAndenForespørgsel() {
        validateSaltReturnedForRequestX(0);
        validateSaltReturnedForRequestX(1);
        assertNotEquals(responses.get(0), responses.get(1));
    }

    @Then("returneres samme salt på alle forespørgsler")
    public void returneresSammeSaltPåAlleForespørgsler() {
        for (int i = 0; i < responses.size(); i++) {
            validateSaltReturnedForRequestX(i);
        }
        long distinctResponses = responses.stream().distinct().count();
        assertEquals(1, distinctResponses);
    }

    private void validateSaltReturnedForRequestX(int requestNumber) {
        assertTrue(requestFailedList.size() > requestNumber);
        assertTrue(responses.size() > requestNumber);
        assertFalse(responses.get(requestNumber), requestFailedList.get(requestNumber));
        assertNotNull(responses.get(requestNumber));
    }

    private void validateRequestXFailed(int requestNumber) {
        assertTrue(requestFailedList.size() > requestNumber);
        assertTrue(requestFailedList.get(requestNumber));
    }

    private void sendGetCurrentSaltRequest(Identity identity) {
        GetCurrentSaltRequest request = new GetCurrentSaltRequest();
        String response;
        boolean requestFailed = false;
        try {
            byte[] r = serviceCaller.callRoot(identity,
                    request, SOAPOperation.GetCurrentSalt, new URI(serviceEndpoint),
                    GetCurrentSaltRequest.class,
                    byte[].class);
            response = Hex.encodeHexString(r);
        } catch (Exception e) {
            response = e.getMessage();
            requestFailed = true;
        }
        responses.add(response);
        requestFailedList.add(requestFailed);
    }
}