package dk.nsp.idsas.test.stepdefinitions;

import dk.nsp.idsas.test.util.HttpClient;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Value;

import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class WsdlStepDefinitions {

    @Value("${idsas.service.endpoint}" + "/20230601")
    private String serviceEndpoint;

    @Value("${idsas.service.endpoint}" + "/service-contract/secure-wsdl/20230601")
    private String serviceSecureWsdlEndpoint;

    @Value("${idsas.service-lookup.endpoint}" + "/20230601")
    private String lookupServiceEndpoint;

    @Value("${idsas.service-lookup.endpoint}" + "/service-contract/secure-wsdl/20230601")
    private String lookupServiceSecureWsdlEndpoint;

    private int responseCode;
    private String responseBody;

    @When("{string} wsdl hentes fra {string}")
    public void getWsdl(String wsdl, String service) throws IOException {
        String endpoint = service.equals("idsas") ? serviceEndpoint : lookupServiceEndpoint;
        String url = String.format("%s/%s?wsdl", endpoint, wsdl);

        HttpClient httpClient = new HttpClient();
        responseBody = httpClient.get(url);
        responseCode = httpClient.getResponseCode();
    }

    @When("{string} wsdl med security hentes fra {string}")
    public void getSecureWsdl(String wsdl, String service) throws IOException {
        String endpoint = service.equals("idsas") ? serviceSecureWsdlEndpoint : lookupServiceSecureWsdlEndpoint;
        String url = String.format("%s/idsas-%s.wsdl", endpoint, wsdl);

        HttpClient httpClient = new HttpClient();
        responseBody = httpClient.get(url);
        responseCode = httpClient.getResponseCode();
    }

    @Then("Returneres http kode {int}")
    public void validateResponseCode(int expectedResponseCode) {
        assertEquals(expectedResponseCode, responseCode);
    }

    @Then("Svaret indeholder wsdl definition")
    public void validateWsdlDefinitionInResponseBody() {
        assertTrue(responseBody.contains("<wsdl:definitions"));
    }

    @Then("Svaret indeholder security header")
    public void validateSecurityHeaderInResponseBody() {
        assertTrue(responseBody.contains("<soap:message name=\"SecurityHeader\""));
    }
}
