package dk.nsp.idsas.setup;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.integrations.sores.SoresClient;
import dk.nsp.idsas.util.health.*;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.http.PoolingHttpClientFactory;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.service.SaltService;
import dk.nsp.idsas.util.AuditLogger;

import org.apache.http.client.HttpClient;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;

import java.time.Duration;

@Configuration
@PropertySources({
        @PropertySource("classpath:/META-INF/maven/dk.nsp.idsas/idsas-web/pom.properties"),
        @PropertySource(value = "idsas.properties", encoding = "ISO-8859-1")
        // Bemærkning: I Java 8 er properties altid i ISO-8859-1, men fremadrettet skulle de gerne kunne være UTF-8.
})
public class WebSetup {

    @Value("${personinformation.url}")
    private String personInformationUrl;

    @Value("${personinformation.errorcount.duration:PT10M}")
    private String personInformationErrorCountDuration;

    @Value("${personinformation.error.tolerance:0}")
    private int personInformationErrorTolerance;

    @Value("${sores.url}")
    private String soresUrl;

    @Value("${sores.errorcount.duration:PT10M}")
    private String soresErrorCountDuration;

    @Bean
    public PoolingHttpClientFactory poolingHttpClientFactory() {
        return new PoolingHttpClientFactory();
    }

    @Bean
    public PersonInformationClient personInformationClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        return new PersonInformationClient(httpClient, personInformationUrl, Duration.parse(personInformationErrorCountDuration));
    }

    @Bean
    public SoresClient soresClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        return new SoresClient(httpClient, soresUrl, Duration.parse(soresErrorCountDuration));
    }

    @Bean
    public BlurringService blurringService(WhitelistDao whitelistDao, BlurringDao blurringDao, AuditLogger auditLogger, PersonInformationClient personInformationClient, SoresClient soresClient) {
        return new BlurringService(whitelistDao, blurringDao, auditLogger, personInformationClient, soresClient);
    }

    @Bean
    public SaltService saltService(SaltDao saltDao, WhitelistDao whitelistDao) {
        return new SaltService(saltDao, whitelistDao);
    }

    @Bean
    public SaltHealthIndicator saltHealthIndicator(SaltDao saltDao) {
        return new SaltHealthIndicator(saltDao);
    }

    @Bean
    public PersonInformationHealthIndicator personInformationHealthIndicator(PersonInformationClient personInformationClient) {
        return new PersonInformationHealthIndicator(personInformationClient, personInformationErrorTolerance);
    }

    @Bean
    public SoresHealthIndicator soresHealthIndicator(SoresClient soresClient) {
        return new SoresHealthIndicator(soresClient, personInformationErrorTolerance);
    }
}
