package dk.nsp.idsas.ws._20230601;

import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.service.SaltService;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.CreateBlurringRequestType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.GetCurrentSaltRequest;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.IdsasServicePortType;
import org.apache.cxf.annotations.SchemaValidation;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;
import java.util.GregorianCalendar;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasService", urlPatterns={"/20230601/service"})
@WebService(serviceName = "IdsasService", wsdlLocation = "wsdl/20230601/idsas-service.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasServicePort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
public class IdsasServiceWS extends IdsasAbstractWS implements IdsasServicePortType {
    private static final Logger logger = Logger.getLogger(IdsasServiceWS.class);

    @Autowired
    private IdsasActorProvider idsasActorProvider;

    @Autowired
    private BlurringService blurringService;

    @Autowired
    private SaltService saltService;

    @Override
    public String createBlurring(CreateBlurringRequestType createBlurringRequest) {
        logger.trace("Enter create blurring");

        try {
            IdsasActor actor = idsasActorProvider.getActor();
            String idClassification = createBlurringRequest.getPatientId().getClassification().value();
            String cpr = createBlurringRequest.getPatientId().getId();
            GregorianCalendar endDateTime = createBlurringRequest.getEndDateTime().toGregorianCalendar();

            blurringService.createBlurring(actor, idClassification, cpr, endDateTime);
        } catch (IdsasException e) {
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage());
        } catch (Exception e) {
            logger.error("Failed to create blurring", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error");
        }

        return "OK";
    }

    @Override
    public byte[] getCurrentSalt(GetCurrentSaltRequest getCurrentSaltRequest) {
        logger.trace("Enter get current salt");

        try {
            IdsasActor actor = idsasActorProvider.getActor();
            return saltService.getCurrentSalt(actor);
        } catch (IdsasException e) {
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage());
        } catch (Exception e) {
            logger.error("Failed to get current salt", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error");
        }
    }
}
